CREATE TABLE business (
	businessid BIGINT GENERATED BY DEFAULT AS IDENTITY(START WITH 1) NOT NULL,
	name VARCHAR(15),
	attention VARCHAR(50),
	address VARCHAR(30),
	city VARCHAR(30),
	state VARCHAR(20),
	zip VARCHAR(10),
	country VARCHAR(20),
	phone VARCHAR(20),
	fax VARCHAR(20),
	email VARCHAR(30),
	PRIMARY KEY (businessid)
);

CREATE TABLE service (
	serviceid BIGINT GENERATED BY DEFAULT AS IDENTITY(START WITH 1) NOT NULL,
	name VARCHAR(15) NOT NULL,
	type INTEGER,
	rate NUMERIC,
	enabled BOOLEAN,
	businessid BIGINT,
	PRIMARY KEY(serviceid),
	CONSTRAINT fk_service_businessid FOREIGN KEY (businessid) REFERENCES PUBLIC.business(businessid)	
);

CREATE TABLE client (
	clientid BIGINT GENERATED BY DEFAULT AS IDENTITY(START WITH 1) NOT NULL,
	name VARCHAR(15) NOT NULL,
	attention VARCHAR(50),
	notes VARCHAR(50),
	address VARCHAR(30),
	city VARCHAR(30),
	state VARCHAR(20),
	zip VARCHAR(10),
	country VARCHAR(20),
	phone VARCHAR(20),
	fax VARCHAR(20),
	email VARCHAR(30),
	businessid BIGINT,
	PRIMARY KEY(clientid),
	CONSTRAINT fk_client_businessid FOREIGN KEY (businessid) REFERENCES PUBLIC.business(businessid)	
);

CREATE TABLE project (
	projectid BIGINT GENERATED BY DEFAULT AS IDENTITY(START WITH 1) NOT NULL,
	name VARCHAR(15) NOT NULL,
	notes VARCHAR(50),
	clientid BIGINT,
	PRIMARY KEY(projectid),
	CONSTRAINT fk_project_clientid FOREIGN KEY (clientid) REFERENCES PUBLIC.client(clientid)	
);

CREATE TABLE invoice (
	invoiceid BIGINT GENERATED BY DEFAULT AS IDENTITY(START WITH 1) NOT NULL,
	number VARCHAR(15) NOT NULL,
	date TIMESTAMP,
	clientid BIGINT,
	PRIMARY KEY(invoiceid),
	CONSTRAINT fk_invoice_clientid FOREIGN KEY (clientid) REFERENCES PUBLIC.client(clientid)	
);

CREATE TABLE invoiceproject (
	invoiceprojectid BIGINT GENERATED BY DEFAULT AS IDENTITY(START WITH 1) NOT NULL,
	invoiceid BIGINT,
	projectid BIGINT,
	PRIMARY KEY(invoiceprojectid),
	CONSTRAINT fk_invoiceproject_projectid FOREIGN KEY (projectid) REFERENCES PUBLIC.project(projectid),
	CONSTRAINT fk_invoiceproject_invoiceid FOREIGN KEY (invoiceid) REFERENCES PUBLIC.invoice(invoiceid)	
);

CREATE TABLE expense (
	expenseid BIGINT GENERATED BY DEFAULT AS IDENTITY(START WITH 1) NOT NULL,
	name VARCHAR(15) NOT NULL,
	description VARCHAR(50),
	amount NUMERIC,
	projectid BIGINT,
	invoiceprojectid BIGINT,
	PRIMARY KEY(expenseid),
	CONSTRAINT fk_expense_projectid FOREIGN KEY (projectid) REFERENCES PUBLIC.project(projectid),
	CONSTRAINT fk_expense_invoiceprojectid FOREIGN KEY (invoiceprojectid) REFERENCES PUBLIC.invoiceproject(invoiceprojectid)
);

CREATE TABLE item (
	itemid BIGINT GENERATED BY DEFAULT AS IDENTITY(START WITH 1) NOT NULL,
	name VARCHAR(15) NOT NULL,
	description VARCHAR(50),
	elapsedtime BIGINT,
	billing INTEGER,
	serviceid BIGINT,
	rate NUMERIC,
	units NUMERIC,
	projectid BIGINT,
	invoiceprojectid BIGINT,
	PRIMARY KEY(itemid),
	CONSTRAINT fk_item_serviceid FOREIGN KEY (serviceid) REFERENCES PUBLIC.service(serviceid),	
	CONSTRAINT fk_item_projectid FOREIGN KEY (projectid) REFERENCES PUBLIC.project(projectid),	
	CONSTRAINT fk_item_invoiceprojectid FOREIGN KEY (invoiceprojectid) REFERENCES PUBLIC.invoiceproject(invoiceprojectid)	
);
